/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/
#include "jsm.h"

/* ldap check - draft module - zad (zad@jabber.org) */
/* ------------------------------------------------ */

#include <stdio.h>
#include <lber.h>
#include <ldap.h>
#include <string.h>

/* to compile nss_ldap-2.1.3 */
/* LDAP connection definitions                   */
/* Adjust these setting for your own LDAP server */
#define HOSTNAME "10.1.1.61"
#define PORT_NUMBER   1004
#define BIND_DN "cn=administrator,ou=members,o=mysite"
#define BIND_PW "password"
#define BASE_DN "ou=members, o=mysite"
#define MAX_UN 36
#define SCOPE LDAP_SCOPE_SUBTREE
#define LDAPCHK_SUCCESS 1 /* TRUE */ 
#define LDAPCHK_FAIL    0 /* FALSE */
#define Z_DEBUG
/* ---------------------------------------- */

/* 
	Purpose: check user un/pw on ldap
*/
short checkLdapAttr(short *cause, char *filter, char *attrName, char *attrValue)
{
   short ret = LDAPCHK_FAIL;
   LDAP *ld;
   LDAPMessage *result, *e;
   char **vals;
   int res;
   int rc;
   char *attrs[] = {attrName, NULL};	// attributes to return
   char *upasswd; 			            // password returned by ldap

	*cause = 0;

   /* Get a handle to an LDAP connection. */
   if ((ld = ldap_init(HOSTNAME, PORT_NUMBER)) == NULL) 
   {  
      /*** ERROR: ldap initialize failed ***/
      *cause = 1;
      return LDAPCHK_FAIL;
   }

   /* Bind anonymously to the LDAP server. */
   if ((rc = ldap_simple_bind_s(ld, BIND_DN, BIND_PW)) != LDAP_SUCCESS) 
   {
     /**** ERROR: ldap bind failed */
     *cause = 2;
      return LDAPCHK_FAIL;
   }
   
   /* Search for the entry. */
   if ((rc = ldap_search_s(ld, BASE_DN, SCOPE, filter, attrs, LDAP_NO_LIMIT, &result)) != LDAP_SUCCESS) 
   {  /**** ERROR: Failed to find the LDAP entry*/
	*cause = 3;
	return LDAPCHK_FAIL;   
   } 

   /* Since we are doing a base search, there should be only one matching entry.  */
   e = ldap_first_entry(ld, result); 
   if (e != NULL) 
   {
		/* Get the attributes  */
         	
		if ((vals = ldap_get_values(ld, e, attrName)) != NULL) 
		{
			upasswd = vals[0];
			res = strncmp(upasswd, attrValue, strlen(attrValue));
			if (res == 0 && (strlen(upasswd) == strlen(attrValue)))
				ret = LDAPCHK_SUCCESS;
			else
			{
			   /**** ERROR: Invalid Password */ 
				*cause = 10;   
				ret = LDAPCHK_FAIL;
			}
		} 
		else 
		{	/**** ERROR: No such attribute exist */
			*cause = 4;
			ret = LDAPCHK_FAIL;
	   } 
		ldap_value_free(vals);	
        }
	else
	{	/*** ERROR: User Not Found*/
		*cause = 11;
		ret = LDAPCHK_FAIL;
		
	}

	ldap_msgfree(result);
   ldap_unbind(ld);

   return ret;
}

/*
	Purpose: Login Authentication. 
		Calls the ldap control routine to heck user on ldap.

*/
mreturn mod_auth_ldap_auth(mapi m, void *arg)
{
   short cause;
   char *in_pw;
   char *in_un;
   char strUname[MAX_UN];
	
   log_debug("mod_auth_ldap_ereg","ldap auth control ... ");

   if (m->packet->type != JPACKET_IQ) 
	return M_IGNORE;
		
	if(jpacket_subtype(m->packet) == JPACKET__GET)
   { /* type=get means we flag that the server can do digest auth */
      xmlnode_insert_tag(m->packet->iq,"password");
      return M_PASS;
   }

   if (jpacket_subtype(m->packet) != JPACKET__SET)
      return M_PASS;
		

   if ((in_un = xmlnode_get_tag_data(m->packet->iq,"username")) == NULL) 
   {
	jutil_error(m->packet->x, TERROR_AUTH);
	return M_HANDLED;
   }
		
   if ((in_pw = xmlnode_get_tag_data(m->packet->iq,"password")) == NULL) 
   {
	jutil_error(m->packet->x, TERROR_AUTH);
	return M_HANDLED;		
   }

	
/*  Check the ldap records */	
   strcpy(strUname, "cn=");
   strcat(strUname, in_un);
	
   if (checkLdapAttr(&cause, strUname, "userpassword", in_pw) == LDAPCHK_SUCCESS)
   {
	log_debug("mod_auth_ldap_eaut","LDAP OK cause=%d", cause);
	jutil_iqresult(m->packet->x);
   }
   else
   {	
	log_debug("mod_auth_ldap_eaut","LDAP NOT FOUND cause=%d", cause);
	jutil_error(m->packet->x, TERROR_AUTH);	
   }

   return M_HANDLED;   
}

/*
	Purpose: Forbid Registering a new user.
	Normally we use this callback to register a new user. But being an ldap auth.
	I am assuming that we are registering users somewhere else, and there is no
	registration through jabber server.
*/

mreturn mod_auth_ldap_noreg(mapi m, void *arg)
{
   
	if (m->packet->type != JPACKET_IQ) 
		return M_IGNORE;
	
	log_debug("mod_auth_ldap_ereg","ldap NO Registration ... ");
		
	if (jpacket_subtype(m->packet) != JPACKET__SET) return M_PASS;

	jutil_error(m->packet->x, TERROR_FORBIDDEN);
	
	return M_HANDLED;
}

/* the main startup/initialization function */
void mod_auth_ldap_z(jsmi si)
{
	js_mapi_register(si, e_REGISTER, mod_auth_ldap_noreg, NULL);
	js_mapi_register(si, e_AUTH, mod_auth_ldap_auth, NULL); 
}
